"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgresIntrospector = void 0;
const migrator_js_1 = require("../../migration/migrator.js");
const object_utils_js_1 = require("../../util/object-utils.js");
class PostgresIntrospector {
    #db;
    constructor(db) {
        this.#db = db;
    }
    async getMetadata(options = { withInternalKyselyTables: false }) {
        let query = this.#db
            .selectFrom('pg_catalog.pg_attribute as a')
            .innerJoin('pg_catalog.pg_class as c', 'a.attrelid', 'c.oid')
            .innerJoin('pg_catalog.pg_tables as t', 't.tablename', 'c.relname')
            .innerJoin('pg_catalog.pg_type as typ', 'a.atttypid', 'typ.oid')
            .select([
            'a.attname as column',
            'a.attnotnull as not_null',
            't.tablename as table',
            't.schemaname as schema',
            'typ.typname as type',
        ])
            .where('t.schemaname', '!~', '^pg_')
            .where('t.schemaname', '!=', 'information_schema')
            .where('a.attnum', '>=', 0) // No system columns
            .where('a.attisdropped', '!=', true)
            .castTo();
        if (!options.withInternalKyselyTables) {
            query = query
                .where('t.tablename', '!=', migrator_js_1.MIGRATION_TABLE)
                .where('t.tablename', '!=', migrator_js_1.MIGRATION_LOCK_TABLE);
        }
        const rawColumns = await query.execute();
        return {
            tables: this.#parseTableMetadata(rawColumns),
        };
    }
    #parseTableMetadata(columns) {
        return columns.reduce((tables, it) => {
            let table = tables.find((tbl) => tbl.name === it.table && tbl.schema === it.schema);
            if (!table) {
                table = (0, object_utils_js_1.freeze)({
                    name: it.table,
                    schema: it.schema,
                    columns: [],
                });
                tables.push(table);
            }
            table.columns.push((0, object_utils_js_1.freeze)({
                name: it.column,
                dataType: it.type,
                isNullable: !it.not_null,
            }));
            return tables;
        }, []);
    }
}
exports.PostgresIntrospector = PostgresIntrospector;
